@@ -1,5 +1,6 @@
 from __future__ import unicode_literals, print_function
 import os
+import re
 import sys
 import six
 import stat
@@ -20,6 +21,7 @@ from ConfigObject import ConfigObject
 from contextlib import contextmanager
 
 __all__ = [
+    'logopts', 'info', 'debug', 'error', 'exc',  # logging
     'console_script', 'requires', 'sh', 'env', 'ini', 'stdin', 'test',
     'ls', 'cat', 'grep', 'find', 'cut', 'tr', 'head', 'tail', 'sed', 'awk',
     'nc', 'ping', 'nmap', 'hostname', 'host', 'scp', 'rsync', 'wget', 'curl',
@@ -43,7 +45,52 @@ aliases = dict(
     ifconfig='/sbin/ifconfig',
     sudo='/usr/bin/sudo',
     ssh='ssh',
-  )
+)
+
+
+class Log(object):
+
+    initialized = False
+    loggers = {}
+
+    def __call__(self, args={'--quiet': False, '--verbose': False},
+                 fmt=None, name=None, stream=sys.stderr):
+
+        if not name:
+            name = posixpath.basename(sys.argv[0])
+
+        if name in self.loggers:
+            return self.loggers[name]
+
+        log = logging.getLogger(name)
+
+        if not log.handlers:
+            if fmt == 'brief':
+                fmt = '[%(levelname)-4s] %(message)s'
+            elif fmt == 'msg':
+                fmt = '%(message)s'
+            else:
+                fmt = '%(asctime)s %(levelname)-6s %(name)s %(message)s'
+            logging.basicConfig(stream=stream, format=fmt)
+
+        if not log.level:  # pragma: no cover
+            if args.get('--quiet'):
+                level = logging.ERROR
+            elif args.get('--debug'):
+                level = logging.DEBUG
+            else:
+                level = logging.INFO
+
+            log.setLevel(level)
+        self.loggers[name] = log
+        return log
+
+    def log(self, name):
+        def wrapper(*args, **kwargs):
+            log = self()
+            return getattr(log, str(name))(*args, **kwargs)
+        wrapper.__name__ = str(name)
+        return wrapper
 
 
 def check_sudo():
@@ -128,6 +175,8 @@ class Pipe(object):
         self.kwargs = kwargs
         if 'sh' in kwargs:
             kwargs['shell'] = kwargs.pop('sh')
+        if kwargs.get('stderr') == 1:
+            kwargs['stderr'] = STDOUT
         if 'combine_stderr' in kwargs:
             kwargs.pop('combine_stderr')
             kwargs['stderr'] = STDOUT
@@ -207,11 +256,8 @@ class Pipe(object):
             args.append(cmd)
         else:
             args.extend(binary.split())
-            if isinstance(self.args, list):
-                for a in self.args:
-                    args.extend(a.split())
-            else:
-                args.extend(self.args.split(" "))
+            for a in self.args:
+                args.extend(a.split())
 
         args = [a for a in args if a]
 
@@ -256,7 +302,7 @@ class Pipe(object):
                 kwargs = dict(
                     stdin=stdin, stderr=PIPE,
                     stdout=PIPE
-                    )
+                )
                 kwargs.update(cmd.kwargs)
                 env_ = kwargs.pop('env', env)
 
@@ -290,7 +336,7 @@ class Pipe(object):
         kw = dict(
             stdin=sys.stdin, stderr=PIPE,
             stdout=PIPE
-            )
+        )
         kw.update(kwargs)
         if pool_size is None:
             import multiprocessing
@@ -324,11 +370,11 @@ class Pipe(object):
                     if result > 0 and stop_on_failure:
                         args = None
                         for index, cmd, p in processes:
-                            if p.poll() is None:
+                            if p.poll() is None:  # pragma: no cover
                                 p.kill()
                         cmd._raise(output=output)
             time.sleep(.1)
-        if out_index < len(results):
+        if out_index < len(results):  # pragma: no cover
             yield results[out_index]
             out_index += 1
 
@@ -345,8 +391,9 @@ class Pipe(object):
         return self._order(cmds)[-1]
 
     def __iter__(self):
+        eol = six.PY3 and '\n' or b'\n'
         for line in self.stdout:
-            yield self._decode(line).rstrip('\n')
+            yield self._decode(line).rstrip(eol)
 
     def __call__(self, **kwargs):
         if self._done and self._stdout is not None:
@@ -356,11 +403,13 @@ class Pipe(object):
                 cmd.kwargs['shell'] = True
             if kwargs.get('combine_stderr'):
                 cmd.kwargs['stderr'] = STDOUT
+            if kwargs.get('stderr'):
+                cmd.kwargs['stderr'] = STDOUT
         stdout = self.stdout
         if stdout is not None:
             if hasattr(stdout, 'read'):
                 output = stdout.read().rstrip()
-            else:
+            else:  # pragma: no cover
                 output = b''.join(list(stdout)).rstrip()
             output = self._decode(output)
         else:
@@ -537,7 +586,8 @@ class Base(object):
     def set_debug(self, enable=True):
         if enable:
             log.setLevel(logging.DEBUG)
-            log.addHandler(logging.StreamHandler(sys.stdout))
+            if not log.handlers:
+                log.addHandler(logging.StreamHandler(sys.stdout))
         else:
             log.setLevel(logging.INFO)
 
@@ -582,16 +632,12 @@ class Chut(Base):
 
     def cd(self, directory):
         """Change the current directory"""
-        if self.__name__ not in ('sh', 'sudo'):
-            raise ImportError('You can only run cd in local commands')
         directory = os.path.realpath(directory)
         os.chdir(directory)
         env.pwd = directory
 
     def pwd(self):
         """return os.path.abspath(os.getcwd())"""
-        if self.__name__ not in ('sh', 'sudo'):
-            raise ImportError('You can only use pwd in local commands')
         return os.path.abspath(os.getcwd())
 
     def stdin(self, value):
@@ -628,6 +674,12 @@ class SSH(Base):
             quote = quote.encode('ascii')
         return host + quote + escape(p) + quote
 
+    def cd(self, *args):
+        raise NotImplementedError('cd does not work with ssh')
+
+    def pwd(self):
+        raise NotImplementedError('pwd does not work with ssh')
+
     @property
     def host(self):
         return self._cmd_args[-1]
@@ -663,7 +715,7 @@ class ModuleWrapper(types.ModuleType):
         if attr == '__all__':
             if self.__name__ == 'chut':
                 return [str(c) for c in __all__]
-            else:
+            else:  # pragma: no cover
                 raise ImportError('You cant import things that does not exist')
         if hasattr(self.mod, attr):
             return getattr(self.mod, attr)
@@ -673,6 +725,13 @@ class ModuleWrapper(types.ModuleType):
     __getitem__ = __getattr__
 
 
+logopts = Log()
+info = logopts.log('info')
+debug = logopts.log('debug')
+error = logopts.log('error')
+warn = logopts.log('warn')
+exc = logopts.log('exception')
+
 env = Environ(os.environ.copy())
 sh = Chut('sh')
 sudo = Chut('sudo', '-s')
@@ -693,39 +752,39 @@ def requires(*requirements, **kwargs):
     """Add extra dependencies in a virtualenv"""
     if '/.tox/' in sys.executable:
         venv = os.path.dirname(os.path.dirname(sys.executable))
-    elif env.virtual_env:
+    elif env.virtual_env:  # pragma: no cover
         venv = env.chut_virtualenv = env.virtual_env
-    else:
+    else:  # pragma: no cover
         venv = os.path.expanduser(kwargs.get('venv', '~/.chut/venv'))
-    if not env.pip_download_cache:
+    if not env.pip_download_cache:  # pragma: no cover
         env.pip_download_cache = os.path.expanduser('~/.chut/cache')
         sh.mkdir('-p', env.pip_download_cache)
     bin_dir = os.path.join(venv, 'bin')
-    if bin_dir not in env.path:
+    if bin_dir not in env.path:  # pragma: no cover
         env.path = [bin_dir] + env.path
     requirements = list(requirements)
     if 'chut' not in requirements:
         requirements.insert(0, 'chut')
-    if not test.d(venv):
+    if not test.d(venv):  # pragma: no cover
         import urllib
         url = 'https://raw.github.com/pypa/virtualenv/master/virtualenv.py'
         urllib.urlretrieve(url, '/tmp/_virtualenv.py')
         sh[sys.executable]('-S /tmp/_virtualenv.py', venv) > 1
         sh.rm('/tmp/_virtualenv*', shell=True)
-        print('Installing %s...' % ', '.join(requirements))
+        info('Installing %s...' % ', '.join(requirements))
         sh.pip('install -qM', *requirements) > 1
     elif env.chut_virtualenv:
         upgrade = '--upgrade' in sys.argv
-        if (env.chut_upgrade or upgrade):
+        if (env.chut_upgrade or upgrade):  # pragma: no cover
             installed = ''
         else:
             installed = str(sh.pip('freeze')).lower()
         requirements = [r for r in requirements if r.lower() not in installed]
-        if requirements:
-            print('Updating %s...' % ', '.join(requirements))
+        if requirements:  # pragma: no cover
+            info('Updating %s...' % ', '.join(requirements))
             sh.pip('install -qM --upgrade', *requirements) > 1
     executable = os.path.join(bin_dir, 'python')
-    if not env.chut_virtualenv:
+    if not env.chut_virtualenv:  # pragma: no cover
         env.chut_virtualenv = venv
         os.execve(executable, [executable] + sys.argv, env)
 
@@ -733,13 +792,23 @@ def requires(*requirements, **kwargs):
 class console_script(object):
     """A decorator to take care of sys.argv via docopt"""
 
+    options = (
+        ('-q, --quiet', 'Quiet (No output)'),
+        ('--debug', 'Debug mode (More output / pdb on failure)'),
+        ('-h, --help', 'Show this help'),
+    )
+
     def __init__(self, *args, **opts):
         self._console_script = True
+        self.logopts = opts.pop('logopts', {})
+        for k in ('fmt', 'stream'):
+            if k in opts:
+                self.logopts[k] = opts.pop(k)
         self.docopts = opts
         self.func = self.doc = None
         self.wraps(args)
 
-    def version(self):
+    def version(self):  # pragma: no cover
         version = getattr(sys.modules['__main__'], 'version', 'unknown')
         print('%s %s' % (self.func.__name__, version))
 
@@ -757,8 +826,23 @@ class console_script(object):
                 doc = 'Usage: %prog'
             name = self.func.__name__.replace('_', '-')
             doc = doc.replace('%prog', name).strip()
+            if '%options' in doc:
+                def options(match):
+                    l = match.groups()[-1]
+                    if l is not None:
+                        fmt = '{0:<%s}{1}\n' % l.strip('-s')
+                    else:
+                        fmt = '{0:<20}{1}\n'
+                    opts = ''
+                    for opt in self.options:
+                        opts += fmt.format(*opt)
+                    return opts
+                doc = re.sub('(%options)(-[0-9]+s)*', options, doc)
             doc = doc.replace('\n    ', '\n')
             self.doc = doc
+
+            if 'name' not in self.logopts:
+                self.logopts['name'] = name
         return self
 
     def main(self, arguments=None):
@@ -767,13 +851,23 @@ class console_script(object):
         if ret:
             self.docopts['argv'] = arguments
         arguments = docopt.docopt(self.doc, **self.docopts)
-        if arguments.get('--version') is True:
+        if arguments.get('--version') is True:  # pragma: no cover
             res = self.version()
         else:
+            logopts(arguments, **self.logopts)
             try:
                 res = self.func(arguments)
-            except KeyboardInterrupt:
+            except KeyboardInterrupt:  # pragma: no cover
                 sys.exit(1)
+            except Exception:  # pragma: no cover
+                if arguments.get('--debug'):
+                    info(('> Entering python debuger. '
+                          'Use h for help, q to quit.'))
+                    import pdb
+                    pdb.post_mortem()
+                    return 1
+                else:
+                    raise
         return res if ret else sys.exit(res)
 
     def __call__(self, *args, **kwargs):
@@ -821,7 +915,7 @@ class Generator(object):
                 'six', 'docopt', 'ConfigObject', sys.modules[__name__]
             ] + list(modules)
             modules = ''.join([self.encode_module(m) for m in modules])
-        else:
+        else:  # pragma: no cover
             # get source from _chut_modules
             modules = ''
             for name, data in _chut_modules:
@@ -841,13 +935,13 @@ class Generator(object):
 
         scripts = []
         loop = '--loop' in sys.argv or '-l' in sys.argv
-        while not os.path.isfile(filename):
+        while not os.path.isfile(filename):  # pragma: no cover
             time.sleep(.1)
         mtime = os.stat(filename)[stat.ST_MTIME]
         for func_name in sorted(set(console_scripts)):
             name = func_name[4:].split('(')[0]
             script = os.path.join(self.dest, name.replace('_', '-'))
-            if os.path.isfile(script) and loop:
+            if os.path.isfile(script) and loop:  # pragma: no cover
                 smtime = os.stat(script)[stat.ST_MTIME]
                 if mtime <= smtime:
                     continue
@@ -865,10 +959,10 @@ class Generator(object):
                     fd.write("\nif __name__ == '__main__':\n    %s()\n" % name)
             executable = sh.chmod('+x', script)
             if executable:
-                print(executable.commands_line)
+                info(executable.commands_line)
                 scripts.append(script)
-            else:
-                print('failed to generate %s' % script)
+            else:  # pragma: no cover
+                error('failed to generate %s' % script)
         return scripts
 
     def __call__(self, location):
