@@ -3,16 +3,16 @@ from chut import *  # noqa
 import sys
 import re
 
-_episode = re.compile(r'S([0-9]+)E([0-9]+)')
+_episode = re.compile(r's([0-9]+)\s*e\s*([0-9]+)')
 
 
 def extract_numbers(f):
-    season, episode = _episode.findall(f)[0]
+    season, episode = _episode.findall(f.lower())[0]
     episode = int(season), int(episode)
     return episode
 
 
-@console_script
+@console_script(fmt='brief')
 def vlserie(args):
     """
     Usage: %prog [options] [<season> <episode>]
@@ -23,21 +23,37 @@ def vlserie(args):
     Store the latest play in ~/.vlserie. So you dont have to remember it
     yourself.
 
-    Require vlc.
+    Require vlc or mplayer.
 
     Options:
-    -l,--latest     Play latest instead of next
-    --loop          Loop over episodes
-    -h              Show this help
+
+    -l, --latest            Play latest instead of next
+    --loop                  Loop over episodes
+    %options
     """
 
+    config = ini('~/.vlserie')
+    config.write()
+
+    player = config.player.binary or 'vlc'
+    debug('Using %s player', player)
+
     def play(filename, episode):
-        cmdline = '-f %s' % e(filename)
+        if player == 'vlc':
+            cmdline = '-f --qt-minimal-view %r' % filename
+        elif player == 'mplayer':
+            cmdline = '-fs %r' % filename
+        else:
+            error('Unknown player %r', player)
+            sys.exit(1)
         srts = find('-regex ".*%s\(x\|E\)%02i.*srt"' % episode, shell=True)
         for srt in sorted(srts):
-            cmdline += ' --sub-file %s' % e(srt)
-        cmd = vlc(cmdline, combine_stderr=True, shell=True)
-        print(repr(cmd))
+            if player == 'vlc':
+                cmdline += ' --sub-file %r' % srt
+            elif player == 'mplayer':
+                cmdline += ' -sub %r' % srt
+        cmd = sh[player](cmdline, combine_stderr=True, shell=True)
+        info(repr(cmd))
         serie.latest = filename
         config.write()
         try:
@@ -47,12 +63,11 @@ def vlserie(args):
         if not args['--loop']:
             sys.exit(0)
 
-    config = ini('~/.vlserie')
-    config.write()
     serie = config[path.abspath('.')]
 
-    filenames = find('. -regex ".*S[0-9]+E[0-9]+.*\(avi\|wmv\|mp4\)"',
-                     shell=True)
+    filenames = find(
+        '. -iregex ".*s[0-9]+\s*e\s*[0-9]+.*\(avi\|wmv\|mkv\|mp4\)"',
+        shell=True)
     filenames = [path.basename(f) for f in filenames]
     filenames = sorted([(extract_numbers(f), f) for f in filenames])
 
