@@ -3,7 +3,7 @@ import chut as sh
 import os
 
 
-@sh.console_script
+@sh.console_script(fmt='msg')
 def chutify(args):
     """
     Usage: %prog [options] [<location>]
@@ -22,21 +22,26 @@ def chutify(args):
 
     Options:
 
-    -h, --help                 Print this help
-    -v, --version              Print script version
-    -l, --loop                 Generate scripts when the source change
-    -s NAME, --section NAME    Use NAME section in .chut [default: chut]
-    --devel                    Install develop scripts in bin/
-    --upgrade                  Upgrade virtualenv dependencies
-    -d DIR, --destination=DIR  Destination [default: dist/scripts]
-    -i X, --interpreter=X      Python interpreter to use [default: python]
+    -v, --version                 Print script version
+    -l, --loop                    Generate scripts when the source change
+    -s NAME, --section NAME       Use NAME section in .chut [default: chut]
+    --devel                       Install develop scripts in bin/
+    --upgrade                     Upgrade virtualenv dependencies
+    -d DIR, --destination=DIR     Destination [default: dist/scripts]
+    -i X, --interpreter=X         Python interpreter to use
+    %options-30s
     """
     config = sh.ini('.chut')
-    if sh.env.git_dir:
+    if sh.env.git_dir:  # pragma: no cover
         cfg = config['githook'] or config[args['--section']]
     else:
         cfg = config[args['--section']]
 
+    interpreter = args['--interpreter'] or cfg.interpreter or 'python'
+    if interpreter in ('2', '3'):  # pragma: no cover
+        interpreter = 'python' + interpreter
+    args['--interpreter'] = interpreter
+
     location = args.get('<location>') or cfg.location or os.getcwd()
     location = os.path.expanduser(location)
 
@@ -49,13 +54,13 @@ def chutify(args):
             filename = __file__.replace('.pyc', '.py')
             script = generator(filename)[0]
             sh.mv(script, hook)
-        else:
+        else:  # pragma: no cover
             # install git hook
             sh.cp(__file__, hook)
         executable = sh.chmod('+x', hook)
         if executable:
             print(executable.commands_line)
-        return
+        return 0
 
     if cfg.destination and args['--destination'] == 'dist/scripts':
         args['--destination'] = cfg.destination
@@ -73,10 +78,10 @@ def chutify(args):
                 binary, args = cmd.split(' ', 1)
                 sh[binary](args) > 2
             else:
-                sh[cmd]()
+                sh[cmd]() > 2
         return scripts
 
-    if args['--loop']:
+    if args['--loop']:  # pragma: no cover
         import time
         while True:
             try:
@@ -86,9 +91,6 @@ def chutify(args):
                 return 0
     else:
         scripts = gen()
-        if sh.env.git_dir:
+        if sh.env.git_dir:  # pragma: no cover
             sh.git('add -f', *scripts) > 1
     return 0
-
-if __name__ == '__main__':
-    chutify()
